package cn.com.qmth.stmms.biz.report.utils.module;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectClassGroup;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectClassGroupService;
import cn.com.qmth.stmms.biz.report.utils.Module;
import cn.com.qmth.stmms.biz.report.utils.QuestionGroup;
import cn.com.qmth.stmms.biz.report.utils.ReportContext;
import cn.com.qmth.stmms.biz.report.utils.unit.BaseCalculatorUnit;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.biz.utils.SpringContextHolder;

public class SubjectClassGroupModule implements Module {

    private Map<String, BaseCalculatorUnit> units;

    private ReportContext context;

    public SubjectClassGroupModule(ReportContext context) {
        this.units = new HashMap<String, BaseCalculatorUnit>();
        this.context = context;
    }

    public void process(ExamStudent student) {
        if (student.isUpload() && !student.isAbsent() && !student.isBreach() && student.getSubject() != null) {
            double totalScore = student.getTotalScore();
            // 遍历客观题
            process(student.getScoreList(true), student.getObjectiveQuestionList(), student.getSubjectCode(),
                    student.getClassName(), totalScore, true);
            // 遍历主观题
            process(student.getScoreList(false), student.getSubjectiveQuestionList(), student.getSubjectCode(),
                    student.getClassName(), totalScore, false);
        }
    }

    private void process(List<ScoreItem> scoreList, List<ExamQuestion> questionList, String subjectCode,
            String className, double totalScore, boolean objective) {
        if (scoreList == null || questionList == null) {
            return;
        }
        Map<Integer, QuestionGroup> map = new HashMap<Integer, QuestionGroup>();
        int scoreCount = scoreList.size();
        for (int i = 0; i < questionList.size(); i++) {
            ExamQuestion question = questionList.get(i);
            if (question.getTotalScore() == null || question.getTotalScore() == 0) {
                continue;
            }
            ScoreItem item = scoreCount > i ? scoreList.get(i) : null;
            QuestionGroup group = map.get(question.getMainNumber());
            if (group == null) {
                group = new QuestionGroup(question.getMainNumber());
                map.put(question.getMainNumber(), group);
            }
            group.incrTotalScore(item != null ? item.getScore() : 0);
            group.incrFullScore(question.getTotalScore());
        }
        for (Entry<Integer, QuestionGroup> entry : map.entrySet()) {
            QuestionGroup group = entry.getValue();
            if (group.getFullScore() == 0) {
                continue;
            }
            findUnit(subjectCode + "\t" + className + "\t" + objective + "\t" + entry.getKey()).process(
                    group.getTotalScore(), group.getFullScore(), totalScore);
        }
    }

    public void save() {
        ReportSubjectClassGroupService service = SpringContextHolder.getBean(ReportSubjectClassGroupService.class);
        for (String key : this.units.keySet()) {
            String s[] = key.split("\t");
            if (s.length > 3) {
                String subjectCode = s[0];
                String className = s[1];
                Boolean objective = Boolean.parseBoolean(s[2]);
                Integer mainNumber = Integer.parseInt(s[3]);
                BaseCalculatorUnit unit = units.get(key);
                ReportSubjectClassGroup r = new ReportSubjectClassGroup();
                r.setExamId(this.context.getExamId());
                r.setSubjectCode(subjectCode);
                r.setClassName(className);
                r.setSubjectName(context.getSubject(subjectCode).getName());
                r.setGroupName(context.getGroup(subjectCode + "\t" + objective + "\t" + mainNumber).getMainTitle());
                r.setGroupNumber(mainNumber);
                r.setTotalScore(unit.fullScore);
                r.setObjective(objective);
                r.setDiscrimination(unit.discrimination);
                service.save(r);
            }
        }
    }

    private BaseCalculatorUnit findUnit(String key) {
        BaseCalculatorUnit unit = units.get(key);
        if (unit == null) {
            unit = new BaseCalculatorUnit();
            units.put(key, unit);
        }
        return unit;
    }

}
